/*
 *   AlignmentAlgorithm.h
 *
 *   Authors: mat and jtr
 */

#ifndef FLEXBAR_ALIGNMENTALGORITHM_H_
#define FLEXBAR_ALIGNMENTALGORITHM_H_

#include <iostream>
#include <string>

#include <seqan/basic.h>
#include <seqan/align.h>


template <typename TString>
class AlignmentAlgorithm {

private:
	
	typedef seqan::Align<seqan::CharString, seqan::ArrayGaps> TAlign;
	typedef seqan::Row<TAlign>::Type TRow;
	typedef seqan::Iterator<TRow>::Type TRowIterator;
	
	// typedef int TValue;
	// typedef seqan::Score<TValue, seqan::ScoreMatrix<seqan::Dna5, seqan::Default> > TScoringScheme;
	// 
	// TScoringScheme scoringScheme;
	seqan::Score<int> *m_score;
	
public:
	
	AlignmentAlgorithm(const int match, const int mismatch, const int gapCost){
		
		using namespace seqan;
		
		// scoringScheme = TScoringScheme(gapCost);
		// 
		// for (unsigned i = 0; i < ValueSize<Dna5>::VALUE; ++i){
		// 	for (unsigned j = 0; j < ValueSize<Dna5>::VALUE; ++j){
		// 		if(i == j || i == 4 || j == 4){
		// 			setScore(scoringScheme, Dna5(i), Dna5(j), match);
		// 		}
		// 		else{
		// 			setScore(scoringScheme, Dna5(i), Dna5(j), mismatch);
		// 		}
		// 	        }
		// }
		
		// // print header line
		// for (unsigned i = 0; i < ValueSize<Dna5>::VALUE; ++i){
		// 	std::cout << "\t" << Dna5(i);
		// }
		// std::cout << std::endl;
		// 
		// 	    // print each row
		// 	    for (unsigned i = 0; i < ValueSize<Dna5>::VALUE; ++i) {
		// 	        std::cout << Dna5(i);
		// 	        for (unsigned j = 0; j < ValueSize<Dna5>::VALUE; ++j) {
		// 	            std::cout << "\t" << score(scoringScheme, Dna5(i), Dna5(j));
		// 	        }
		// 	        std::cout << std::endl;
		// 	    }
		
		m_score = new seqan::Score<int>(match, mismatch, gapCost);
	};
	
	
	virtual ~AlignmentAlgorithm(){
		delete m_score;
	};
	
	
	void align(const TString &querySeq, const TString &read, int &gapsR, int &gapsA, int &mismatches, int &startPos, int &endPos, int &startPosA, int &endPosA, int &startPosS, int &endPosS, int &aliScore, std::stringstream &aliString, const flexbar::TrimEnd trimEnd, const flexbar::LogLevel m_verb){
		
		using namespace std;
		using namespace seqan;
		
		TAlign align;
		resize(rows(align), 2);
		assignSource(row(align, 0), read);
		assignSource(row(align, 1), querySeq);
		// assignSource(row(align, 0), (Dna5String) read);
		// assignSource(row(align, 1), (Dna5String) querySeq);
		
		
		if(trimEnd == flexbar::RIGHT || trimEnd == flexbar::RIGHT_TAIL){
			AlignConfig<true, false, true, true> ac;
			aliScore = globalAlignment(align, *m_score, ac, NeedlemanWunsch());
			// globalAlignment(align, *m_score, ac, 2, 2, NeedlemanWunsch());  // MyersHirschberg
		}
		else if(trimEnd == flexbar::LEFT  || trimEnd == flexbar::LEFT_TAIL){
			AlignConfig<true, true, false, true> ac;
			aliScore = globalAlignment(align, *m_score, ac, NeedlemanWunsch());
		}
		else{
			AlignConfig<true, true, true, true> ac;
			aliScore = globalAlignment(align, *m_score, ac, NeedlemanWunsch());
		}
		
		
		TRow &row1 = row(align,0);
		TRow &row2 = row(align,1);
		
		startPosS = toViewPosition(row1, 0);
		startPosA = toViewPosition(row2, 0);
		endPosS   = toViewPosition(row1, length(source(row1)));
		endPosA   = toViewPosition(row2, length(source(row2)));
		
		// calculate overlap start and end
		if(startPosA > startPosS) startPos = startPosA;
		else                      startPos = startPosS;
		
		if(endPosA > endPosS) endPos = endPosS;
		else                  endPos = endPosA;
		
		
		// cout << endl << endl << startPosS << endl << startPosA << endl<< endPosS << endl << endPosA;
		
		// int fstartPosS = toViewPosition(row1, 0);
		// int fstartPosA = toViewPosition(row2, 0);
		// int fendPosS   = toViewPosition(row1, length(source(row1)));
		// int fendPosA   = toViewPosition(row2, length(source(row2)));
		// 
		// cout << endl << endl << fstartPosS << endl << fstartPosA << endl<< fendPosS << endl << fendPosA;
		
		
		// cout << align << endl << aliScore << endl;
		
		if(m_verb != flexbar::NONE) aliString << align;
		
		
		// compute number of mismatches and gaps
		TRowIterator it1 = begin(row1);
		TRowIterator it2 = begin(row2);
		
		int aliPos = 0;
		gapsR      = 0;
		gapsA      = 0;
		mismatches = 0;
		
		for(; it1 != end(row1); ++it1){
			if(startPos <= aliPos && aliPos < endPos){
				
				     if(isGap(it1))          ++gapsR;
				else if(isGap(it2))          ++gapsA;
				else if(*it1 != *it2)        ++mismatches;
				// else if(*it1 != *it2 && *it2 != 'N')        ++mismatches;
			}
			++aliPos;
			++it2;
		}
		
		// cout << endl << endl << gapsR << endl << gapsA << endl<< mismatches << endl << align;
	}
	
};

#endif /* FLEXBAR_ALIGNMENTALGORITHM_H_ */
